@@ -12,7 +12,7 @@ from mindsdb_native.libs.controllers.transaction import (
     LearnTransaction, PredictTransaction
 )
 from mindsdb_native.libs.constants.mindsdb import *
-from mindsdb_native.libs.helpers.general_helpers import check_for_updates, load_lmd, load_hmd
+from mindsdb_native.libs.helpers.general_helpers import load_lmd, load_hmd
 from mindsdb_native.libs.helpers.locking import MDBLock
 from mindsdb_native.libs.helpers.stats_helpers import sample_data
 
@@ -93,10 +93,14 @@ class Predictor:
         """
         self.name = name
         self.uuid = str(uuid.uuid1())
-        if CONFIG.CHECK_FOR_UPDATES:
-            self.report_uuid = check_for_updates(run_env)
-        else:
-            self.report_uuid = 'no_report'
+        # Wrap in try catch since we aren't running this in the CI
+        self.report_uuid = 'no_report'
+        try:
+            from mindsdb_native.libs.helpers.general_helpers import check_for_updates
+            if CONFIG.CHECK_FOR_UPDATES:
+                self.report_uuid = check_for_updates(run_env)
+        except Exception as e:
+            print(e)
         self.log = MindsdbLogger(log_level=log_level, uuid=self.uuid, report_uuid=self.report_uuid)
         self.breakpoint = None
         self.transaction = None
