@@ -13,17 +13,20 @@ from mindsdb_native.libs.data_types.data_source import DataSource
 from mindsdb_native.libs.data_types.mindsdb_logger import log
 
 
+def clean_row(row):
+    n_row = []
+    for cell in row:
+        if str(cell) in ['', ' ', '  ', 'NaN', 'nan', 'NA']:
+            n_row.append(None)
+        else:
+            n_row.append(cell)
+
+    return n_row
+
 class FileDS(DataSource):
-
-    def cleanRow(self, row):
-        n_row = []
-        for cell in row:
-            if str(cell) in ['', ' ', '  ', 'NaN', 'nan', 'NA']:
-                n_row.append(None)
-            else:
-                n_row.append(cell)
-
-        return n_row
+    def __init__(self, *args, **kwargs):
+        self.is_sql = False
+        super(FileDS, self).__init__(*args, **kwargs)
 
     def _getDataIo(self, file):
         """
@@ -49,7 +52,8 @@ class FileDS(DataSource):
         # else read file from local file system
         else:
             try:
-                data = open(file, 'rb')
+                with open(file, 'rb') as fp:
+                    data = BytesIO(fp.read())
             except Exception as e:
                 error = 'Could not load file, possible exception : {exception}'.format(exception = e)
                 log.error(error)
@@ -146,9 +150,6 @@ class FileDS(DataSource):
             # No file type identified
             return data, None, dialect
 
-    def name(self):
-        return '{}: {}'.format(self.__class__.__name__, self._file_name)
-
     def _setup(self, file, clean_rows=True, custom_parser=None):
         """
         Setup from file
@@ -182,12 +183,12 @@ class FileDS(DataSource):
             df = json_normalize(json_doc)
             header = df.columns.values.tolist()
             file_data = df.values.tolist()
-        
+
         else:
             raise ValueError('Could not load file into any format, supported formats are csv, json, xls, xlsx')
 
         if clean_rows == True:
-            file_list_data = [self.cleanRow(row) for row in file_data]
+            file_list_data = [clean_row(row) for row in file_data]
         else:
             file_list_data = file_data
 
