@@ -1,13 +1,16 @@
 import os
 
-import pandas as pd 
-import pg8000 
+import pandas as pd
+import pg8000
 
 from mindsdb_native.libs.data_types.data_source import DataSource
 
 
 class RedshiftDS(DataSource):
-    
+    def __init__(self, *args, **kwargs):
+        self.is_sql = True
+        super(RedshiftDS, self).__init__(*args, **kwargs)
+
     def _setup(self, table=None, query=None, database='dev', host='localhost',
                port=5439, user='awsuser', password=''):
 
@@ -30,7 +33,7 @@ class RedshiftDS(DataSource):
         col_map = {}
         for col in df.columns:
             col_map[col] = col
-        
+
         return df, col_map
 
     def name(self):
@@ -38,4 +41,4 @@ class RedshiftDS(DataSource):
             self.__class__.__name__,
             self._database_name,
             self._table_name
-        )
\ No newline at end of file
+        )
