@@ -21,6 +21,13 @@ from mindsdb_native.libs.helpers.text_helpers import (
     shrink_word_dist
 )
 
+from mindsdb_native.libs.constants.mindsdb import (
+    DATA_TYPES,
+    DATA_SUBTYPES,
+    DATA_TYPES_SUBTYPES,
+    DATA_TYPE_ALIASES
+)
+
 
 def lof_outliers(col_subtype, col_data):
     lof = LocalOutlierFactor(contamination='auto')
@@ -114,10 +121,7 @@ def get_image_histogram(data):
     for index in indices:
         y[index] += 1
 
-    return {
-               'x': x,
-               'y': y
-           }, list(kmeans.cluster_centers_)
+    return {'x': x, 'y': y}, list(kmeans.cluster_centers_)
 
 
 def get_histogram(data, data_type, data_subtype):
@@ -337,6 +341,10 @@ class DataAnalyzer(BaseModule):
                     stats_v2[col_name]['guess_probability'] = sum((k / len(col_data))**2 for k in histogram['y'])
                     stats_v2[col_name]['balanced_guess_probability'] = 1 / len(histogram['y'])
 
+            if data_type == DATA_TYPES.CATEGORICAL:
+                if DATA_TYPES.NUMERIC in stats_v2[col_name]['additional_info']['other_potential_types']:
+                    stats_v2[col_name]['typing']['alias'] = DATA_TYPE_ALIASES.NUMERICAL_LOW_GRANULARITY
+
         self.transaction.lmd['data_preparation']['accepted_margin_of_error'] = self.transaction.lmd['sample_settings']['sample_margin_of_error']
 
         self.transaction.lmd['data_preparation']['total_row_count'] = len(input_data.data_frame)
