@@ -15,7 +15,7 @@ class TestMYSQL(unittest.TestCase):
     def test_mysql_ds(self):
         from mindsdb_native import MySqlDS
 
-        LIMIT = 100
+        LIMIT = 400
 
         mysql_ds = MySqlDS(
             table=self.TABLE,
@@ -24,7 +24,7 @@ class TestMYSQL(unittest.TestCase):
             password=self.PASSWORD,
             database=self.DATABASE,
             port=self.PORT,
-            query='SELECT * FROM {} LIMIT {}'.format(self.TABLE, LIMIT)
+            query=' (SELECT * FROM (SELECT * FROM {table} LIMIT {limit}) as t1) UNION ALL (SELECT * FROM (SELECT * FROM {table} LIMIT {limit}) as t1)'.format(table=self.TABLE, limit=int(LIMIT/2))
         )
 
         mysql_ds.df = break_dataset(mysql_ds.df)
@@ -32,3 +32,12 @@ class TestMYSQL(unittest.TestCase):
         assert len(mysql_ds) <= LIMIT
 
         F.analyse_dataset(mysql_ds)
+
+        # Our SQL parsing fails here, test if we're still able to filter via the dataframe fallback
+        assert mysql_ds.is_sql
+        for val in mysql_ds.filter([['sex', 'like','fem']])['sex']:
+            assert val == 'female'
+
+        assert len(mysql_ds.filter([['age', '>', 20]], 12)) == 12
+        assert len(mysql_ds.filter([['age', '=', 60]], 1)) == 1
+        assert len(mysql_ds.filter([['age', '>', 150]], 11)) == 0
